# -*- coding: utf-8 -*-
from PyQt4.QtGui import *
from PyQt4.QtCore import *
from pyObjectE import ytfctrl
from PyQt4.QtMaemo5 import QMaemo5InformationBox
from Ui_youtubewindows import Ui_YoutubeWindows
from videoinfowindow import VideoInfoWindow
import delegatewidgets
import youtubeservice
import youtubedialogs
from youtubeconfig import GeneralSettings, YouTubeAPISettings, SearchSettings
from videoplaybackwindow import VideoPlaybackWindow
from PyQt4.QtWebKit import *
from PyQt4.QtNetwork import QNetworkAccessManager

class YoutubeWindows(QMainWindow, Ui_YoutubeWindows):
    def __init__(self, parent):
        QMainWindow.__init__(self, parent)
        self.setupUi(self)
        self.setAttribute(Qt.WA_Maemo5StackedWindow)
        self.setAttribute(Qt.WA_DeleteOnClose)
        self.setAttribute(Qt.WA_Maemo5AutoOrientation, GeneralSettings.autoOrientation)
        self.listWidget.keyPressEvent = self.filterList
        self.lineFilter.keyPressEvent = self.filterList
        self.listWidget.setContextMenuPolicy(Qt.ActionsContextMenu)
        self.sublistWidget.setContextMenuPolicy(Qt.ActionsContextMenu)
        queueLength = len(youtubeservice.YouTubeQueueFeed.playlist)
        if queueLength > 0:
            self.actionGoToQueue.setVisible(True)
            self.actionGoToQueue.setText("Queue (%d videos)" % queueLength)
        taskQueueLength = len(youtubeservice.YouTubeVideoDownloader.taskQueue)
        if taskQueueLength > 0:
            self.actionDownloads.setVisible(True)
            self.actionDownloads.setText("Downloads (%d videos)" % taskQueueLength)
        self.connect(self.cFilter, SIGNAL("triggered()"), self.closeFilter)
        self.connect(self.actionAddToQueue, SIGNAL("triggered()"), self.addToQueue)
        self.connect(self.actionPlaying, SIGNAL("triggered()"), self.playingVideo)
        self.connect(self.actionAddToPlaylist, SIGNAL("triggered()"), self.addToPlaylist)
        self.connect(self.actionAddToFavourites, SIGNAL("triggered()"), self.addToFavourites)
        self.connect(self.actionDeleteVideo, SIGNAL("triggered()"), self.confirmVideoDelete)
        self.connect(self.moreButton, SIGNAL("pressed()"), self.changeButtonStyleSheet)
        self.connect(self.moreButton, SIGNAL("released()"), self.changeButtonStyleSheet)        
        self.connect(QApplication.desktop(), SIGNAL("resized(int)"), self.orientationChanged)
        self.yt = None
        self.subyt = None
        self.videoinfo = None
        self.item_index = 0
        self.clearwidgets = False
        self.info = False
        QWebSettings.globalSettings().setAttribute(QWebSettings.PluginsEnabled, True)
        QWebSettings.globalSettings().setAttribute(QWebSettings.JavascriptEnabled, True)
        self.manager = QNetworkAccessManager(self)
        self.flashView = QWebView(self)
        self.flashView.setUrl(QUrl("about:blank"))
        self.flashView.setAutoFillBackground(True)
        self.flashView.setStyleSheet("background-color: rpg(0, 0, 0, 0);")
        self.flashView.setVisible(False)
        self.layout.insertWidget(0, self.flashView)
        self.orientationChanged()

    def closeEvent(self,  event):
        if self.yt != None:          
            while self.yt.isRunning():
                    self.yt.wait()
                    
        if self.subyt != None:
            while self.subyt.isRunning():
                self.subyt.wait()
                
        if self.flashView.isVisible():
            self.stopFlash()
        
    def changeButtonStyleSheet(self):
        button = self.sender()
        if button.isDown():
            self.moreButton.setStyleSheet("font-size: 18px; border-radius: 4px; background-color: rgb(120, 120, 120, 150)")
        else:
            self.moreButton.setStyleSheet("font-size: 18px; border-radius: 4px; background-color: rgb(0, 0, 0, 100)")
            
    def orientationChanged(self):
        screenGeometry = QApplication.desktop().screenGeometry()
        if screenGeometry.height() > screenGeometry.width():
            self.moreButton.setGeometry(QRect(402, 670, 85, 80))
            self.loadingLabel.setGeometry(QRect(0, 722, 170, 25))
            self.noresultsLabel.setGeometry(QRect(0, 280, 480, 200))
            self.closeFilter()
            self.playButton.setGeometry(QRect(245, 103, 64, 64))
            self.pauseButton.setGeometry(QRect(245, 103, 64, 64))
            self.stopButton.setGeometry(QRect(171, 103, 64, 64))
            self.flashView.setFixedSize(480, 270)
            if self.flashView.isVisible():
                self.showMaximized()                
        else:
            self.moreButton.setGeometry(QRect(722, 352, 85, 80))
            self.loadingLabel.setGeometry(QRect(0, 402, 170, 25))
            self.noresultsLabel.setGeometry(QRect(0, 90, 800, 200))
            self.playButton.setGeometry(QRect(405, 192, 96, 96))
            self.pauseButton.setGeometry(QRect(405, 192, 96, 96))
            self.stopButton.setGeometry(QRect(299, 192, 96, 96))
            self.flashView.setFixedSize(800, 480)
            if self.flashView.isVisible():
                self.moreButton.setVisible(False)
                self.showFullScreen()

    def filterList(self, event):
        self.lineFilter.setFocus()
        if event.key() not in (Qt.Key_Left, Qt.Key_Right, Qt.Key_Up, Qt.Key_Down, Qt.Key_Backspace, Qt.Key_Return, Qt.Key_Control):
            self.toolBar.setVisible(True)
            self.moreButton.setVisible(False)
            self.lineFilter.insert(event.text())
        if event.key() == Qt.Key_Backspace:
            self.lineFilter.backspace()
        self.shlistWidget.clear()
        for item in SearchSettings.recentSearches:
            if item.contains(self.lineFilter.text(), Qt.CaseInsensitive):
                a = QListWidgetItem("   %s" % item.replace("+"," "))
                self.shlistWidget.addItem(a)
        self.shlistWidget.setVisible(True)
        position = 360-(70*self.shlistWidget.count())
        height = 70*self.shlistWidget.count()
        if height > 350:
            height = 350
        if position < 1:
            position = 0
        self.shlistWidget.setGeometry(QRect(17, position, 698, height))
        if self.lineFilter.text() == "":
            self.closeFilter()
        if event.key() == Qt.Key_Enter and self.lineFilter.text() != "":
            self.searchQuery(self.lineFilter.text()[:-1], "relevance")
            self.closeFilter()
                    
    def closeFilter(self):
        self.lineFilter.setText("")
        self.toolBar.setVisible(False)
        self.shlistWidget.setVisible(False)
        self.listWidget.setFocus()
        
    @pyqtSignature("QListWidgetItem*")
    def on_shlistWidget_itemClicked(self):
        self.searchQuery(self.shlistWidget.currentItem().text()[3:], "relevance")
        self.closeFilter()

    def toggleBusy(self, isBusy=False, subyt=False):
        self.setAttribute(Qt.WA_Maemo5ShowProgressIndicator, isBusy)
        self.loadingLabel.setVisible(isBusy)
        if not (self.info or isBusy or subyt):
            if self.yt.results == 0:
                self.sublistWidget.clear()
                self.listWidget.clear()
                self.noresultsLabel.setVisible(True)
            self.info = True

    def addToQueue(self):
        row = self.listWidget.currentRow()
        video = self.yt.videoList[row]
        videoTitle = video.media.title.text
        try:
            youtubeservice.YouTubeQueueFeed.playlist.append(video)
            QMaemo5InformationBox.information(self, "\'%s\' added to your playback queue" % unicode(videoTitle, "utf-8"))
            queueLength = len(youtubeservice.YouTubeQueueFeed.playlist)
            self.queueChanged(queueLength)
        except:
            QMaemo5InformationBox.information(self, "Unable to add video to your playback queue")
        self.listWidget.setCurrentRow(-1)

    def showFlash(self):
        self.webPage = QWebPage()
        self.webPage.setNetworkAccessManager(self.manager)
        self.webFrame = self.webPage.mainFrame()
        if "VEVO" in self.videoinfo.media.credit.text or self.videoinfo.control == None:
            flashUrl = youtubeservice.YouTubeDataService.getVideoUrl(self.videoinfo.media.video_id.text, False)
            if flashUrl == "":
                if self.flashView.isVisible():
                    self.stopFlash()
                self.videoPlaybackWindow = VideoPlaybackWindow(self, video=self.videoinfo)
                self.connect(self.videoPlaybackWindow, SIGNAL("dataLoaded(PyQt_PyObject)"), self.videoinfWindow)
                self.connect(self.videoPlaybackWindow, SIGNAL("downloadsChanged(int)"), self.downloadsChanged)                
            else:
                jsscript = "<script type='text/javascript'>function onYouTubePlayerReady(playerId){ytplayer=document.getElementById('myytplayer');setInterval(a, 1000);a();ytplayer.setPlaybackQuality('small');ytplayer.addEventListener('onStateChange', 'onytplayerStateChange');ytplayer.addEventListener('onError', 'pyqtObject.stop');}function onytplayerStateChange(newState){pyqtObject.stop(newState);}function a(){if(ytplayer){var b=ytplayer.getCurrentTime();pyqtObject.prupdate(b);}}</script>"
                html = "<html>" + jsscript + "<body topMargin=\"0\" leftMargin=\"0\" scroll=\"no\"><embed id=\"myytplayer\" type=\"application/x-shockwave-flash\" width=\"100%\" height=\"100%\" src=\"http://www.youtube.com/apiplayer?enablejsapi=1&playerapiid=ytplayer&version=3&autoplay=1\" flashvars=\"url_encoded_fmt_stream_map=url%3Dhttp%253A%252F%252F" + flashUrl + "&fmt_list=5%2F320x240%2F7%2F0%2F0&video_id=" + self.videoinfo.media.video_id.text + "\" allowscriptaccess=\"always\" allowfullscreen=\"true\" bgcolor=\"#000000\"></embed></body></html>"
                flashcontrol = ytfctrl()
                self.connect(flashcontrol, SIGNAL("videostop(int)"), self.ytstatus)
                self.connect(flashcontrol, SIGNAL("duration(int)"), self.ytprupdate)
                self.webFrame.addToJavaScriptWindowObject("pyqtObject", flashcontrol)
                self.webFrame.setHtml(html, QUrl("http://www.youtube.com"))
                self.loadFinished()
        else:
            if self.flashView.isVisible():
                self.stopFlash()
            self.videoPlaybackWindow = VideoPlaybackWindow(self, video=self.videoinfo)
            self.connect(self.videoPlaybackWindow, SIGNAL("dataLoaded(PyQt_PyObject)"), self.videoinfWindow)
            self.connect(self.videoPlaybackWindow, SIGNAL("downloadsChanged(int)"), self.downloadsChanged)
                    
    def loadFinished(self):
        self.flashView.setPage(self.webPage)
        self.flashView.setVisible(True)
        self.orientationChanged()
        self.flashView.mousePressEvent = self.showytcontrol
        self.ytpprogress.setRange(0, float(self.videoinfo.media.duration.seconds))
        self.currentvideoduration = float(self.videoinfo.media.duration.seconds)
        self.ytpprogress.setSliderPosition(0)
        self.connect(self.ytpprogress, SIGNAL("sliderReleased()"), self.playposition)

    def playposition(self):
        self.pauseButton.setVisible(True)
        self.pauseButton.setEnabled(True)
        self.playButton.setVisible(False)
        self.playButton.setEnabled(False)
        self.webFrame.evaluateJavaScript("ytplayer.seekTo(%s, true)" % self.ytpprogress.value())
        
    def ytprupdate(self, a):
        self.ytpprogress.setValue(a)
        if (self.currentvideoduration - a <= 1):
            if self.flashView.isVisible():
                self.stopFlash()
            
    def showytcontrol(self, event=None):
        self.ytpprogress.setVisible(not self.ytpprogress.isVisible())
        self.playButton.setVisible((not self.playButton.isVisible() and self.playButton.isEnabled()))
        self.pauseButton.setVisible((not self.pauseButton.isVisible() and self.pauseButton.isEnabled()))
        self.stopButton.setVisible(not self.stopButton.isVisible())
        self.orientationChanged()        
                
    def ytstatus(self, a):
        if a in (101, 150):
            if self.flashView.isVisible():
                self.stopFlash()
            self.videoPlaybackWindow = VideoPlaybackWindow(self, video=self.videoinfo)
            self.connect(self.videoPlaybackWindow, SIGNAL("dataLoaded(PyQt_PyObject)"), self.videoinfWindow)
            self.connect(self.videoPlaybackWindow, SIGNAL("downloadsChanged(int)"), self.downloadsChanged)
        elif a == 2:
            self.playButton.setEnabled(True)
            self.pauseButton.setEnabled(False)
        elif a == 1:
            self.playButton.setEnabled(False)
            self.pauseButton.setEnabled(True)
        elif a == 0:
            self.stopFlash()
            
    @pyqtSignature("")
    def on_pauseButton_clicked(self):
        self.webFrame.evaluateJavaScript("ytplayer.stopVideo()")
        self.pauseButton.setVisible(False)
        self.pauseButton.setEnabled(False)
        self.playButton.setVisible(True)
        self.playButton.setEnabled(True)        

    @pyqtSignature("")
    def on_playButton_clicked(self):
        self.webFrame.evaluateJavaScript("ytplayer.playVideo()")
        self.pauseButton.setVisible(True)
        self.pauseButton.setEnabled(True)
        self.playButton.setVisible(False)
        self.playButton.setEnabled(False)

    @pyqtSignature("")
    def on_stopButton_clicked(self):
        self.webFrame.evaluateJavaScript("ytplayer.stopVideo()")
        self.stopFlash()
        
    def stopFlash(self, event=None):
        self.flashView.load(QUrl("about:blank"))
        self.ytpprogress.setVisible(False)
        self.flashView.setVisible(False)
        self.playButton.setVisible(False)
        self.pauseButton.setVisible(False)
        self.stopButton.setVisible(False)
        self.showMaximized()
        self.orientationChanged()
        self.listWidget.setFocus()

    def playingVideo(self, event=None):
        self.toggleBusy(True)
        row = self.listWidget.currentRow()
        self.videoinfo = self.yt.videoList[row]
        screenGeometry = QApplication.desktop().screenGeometry()
        if screenGeometry.height() > screenGeometry.width():
            try:
                if self.videoPlaybackWindow.player.isPlaying():
                    self.videoPlaybackWindow.close()
            except:
                pass
            self.showFlash()  
        else:
            try:
                if self.videoPlaybackWindow.player.isPlaying():
                    self.videoPlaybackWindow.close()
            except:
                pass
            
            try:
                if self.flashView.isVisible():
                    self.stopFlash()
                self.videoPlaybackWindow = VideoPlaybackWindow(self, video=self.videoinfo)
                self.connect(self.videoPlaybackWindow, SIGNAL("dataLoaded(PyQt_PyObject)"), self.videoinfWindow)
                self.connect(self.videoPlaybackWindow, SIGNAL("downloadsChanged(int)"), self.downloadsChanged)
            except:
                QMaemo5InformationBox.information(self, "Unable to play video")
            self.listWidget.setCurrentRow(-1)
        self.toggleBusy()
        
    def playingVideo1(self, video):
        self.videoinfo = video
        screenGeometry = QApplication.desktop().screenGeometry()
        if screenGeometry.height() > screenGeometry.width(): 
            try:
                if self.videoPlaybackWindow.player.isPlaying():
                    self.videoPlaybackWindow.close()              
            except:
                pass
            self.showFlash()
        else:
            try:
                if self.videoPlaybackWindow.player.isPlaying():
                    self.videoPlaybackWindow.close()              
            except:
                pass
            
            try:
                if self.flashView.isVisible():
                    self.stopFlash()
                self.videoPlaybackWindow = VideoPlaybackWindow(self, video=self.videoinfo)
                self.connect(self.videoPlaybackWindow, SIGNAL("dataLoaded(PyQt_PyObject)"), self.videoinfWindow)
                self.connect(self.videoPlaybackWindow, SIGNAL("downloadsChanged(int)"), self.downloadsChanged)
            except:
                QMaemo5InformationBox.information(self, "Unable to play video")

    def videoinfWindow(self, video):
        videoInfoWindow = VideoInfoWindow(self, video)
        self.connect(videoInfoWindow, SIGNAL("dataLoaded(PyQt_PyObject)"), self.playingVideo1)
        self.connect(videoInfoWindow, SIGNAL("queueChanged(int)"), self.queueChanged)
        self.connect(videoInfoWindow, SIGNAL("downloadsChanged(int)"), self.downloadsChanged)
        self.listWidget.setCurrentRow(-1)
        self.actionPencil.setChecked(False)
        self.on_actionPencil_toggled(False)
        self.closeFilter()
        self.pencilList = []        
        
    def addToPlaylist(self):
        row = self.listWidget.currentRow()
        video = self.yt.videoList[row]
        addToPlaylistDialog = youtubedialogs.AddToPlaylistDialog(self, [video])
        self.listWidget.setCurrentRow(-1)

    def addToFavourites(self):
        row = self.listWidget.currentRow()
        video = self.yt.videoList[row]
        videoId = video.media.video_id.text
        try:
            addedToFavourites = youtubeservice.YouTubeDataService.addVideoToFavourites(videoId)
            if addedToFavourites:
                QMaemo5InformationBox.information(self, "\'%s\' has been added to your favorites" % unicode(video.media.title.text, "utf-8"))
            else:
                QMaemo5InformationBox.information(self, addedToFavourites)
        except:
            QMaemo5InformationBox.information(self, "Unable to add video to your favorites")
        self.listWidget.setCurrentRow(-1)

    def confirmVideoDelete(self):
        self.row = self.listWidget.currentRow()
        self.video = self.yt.videoList[self.row]
        confirmDeleteDialog = youtubedialogs.ConfirmDeleteDialog(self, "%s" % self.video.media.title.text)
        self.connect(confirmDeleteDialog, SIGNAL("deleteConfirmed()"), self.deleteVideo)
        self.listWidget.setCurrentRow(-1)

    def deleteVideo(self):
        for link in self.video.link:
            if link.rel == "self":
                video_uri = link.href
        try:
            youtubeservice.YouTubeDataService.deleteVideo(video_uri)
            self.lastitemcheck()
            self.reloadData()
        except:
            QMaemo5InformationBox.information(self, "Unable to delete video from your channel")
        self.listWidget.setCurrentRow(-1)

    def loadData(self, subyt=False):
        try:
            if subyt:
                self.subyt.start()
            else:
                self.yt.start()
        except:
            try:
                QMaemo5InformationBox.information(self, "Unable to connect to YouTube, please try again")
            except:
                pass
        
    def getMoreResults(self, scrollBarPos):
        if self.yt.isFinished() and self.yt.moreResults and not self.isFullScreen() and scrollBarPos > (self.yt.results - 10) * 120:
            self.moreButton.setVisible(True)
        else:
            self.moreButton.setVisible(False)
            
    @pyqtSignature("")
    def on_moreButton_clicked(self):
        self.loadData()
        self.moreButton.setVisible(False)

    def reloadData(self, subyt=False):
        self.clearwidgets = True
        self.closeFilter()
        if not subyt:
            self.yt.resetResultCount()
        self.loadData(subyt)

    def addVideo(self, videoData):
        if self.clearwidgets:
            self.listWidget.clear()
            self.sublistWidget.setVisible(False)
            self.listWidget.setVisible(True)
            self.listWidget.setFocus()
            self.clearwidgets = False
        size = QSize()
        size.setHeight(140)
        item = QListWidgetItem(self.listWidget)
        item.setSizeHint(size)
        itemWidget = delegatewidgets.VideoItemWidget(self.listWidget, videoData)
        self.item_index += 1
        if self.item_index % 2 == 0:
            itemWidget.setStyleSheet("background-color: rgb(64, 64, 64, 100);")
        if self.actionPencil.isChecked():
            itemWidget.showCheckBox()
            item.setFlags(Qt.ItemIsEnabled)
        self.listWidget.addItem(item)
        self.listWidget.setItemWidget(item, itemWidget)
        title = itemWidget.titleLabel.text()
        author = itemWidget.authorLabel.text()
        if not title.contains(self.lineFilter.text(), Qt.CaseInsensitive) and not author.contains(self.lineFilter.text(), Qt.CaseInsensitive):
            self.listWidget.setItemHidden(item, True)
        if not self.info:
            self.info = True
                
    def addVideoThumbnail(self, videoData):
        row, thumbnail = videoData
        item = self.listWidget.item(row)
        thumbLabel = self.listWidget.itemWidget(item).thumbLabel
        try:
            thumbLabel.setPixmap(thumbnail)
            thumbLabel.mouseReleaseEvent = self.playingVideo
        except:
            thumbLabel.setPixmap(QPixmap(":/images/ui-images/thumbnail.png"))

    @pyqtSignature("QListWidgetItem*")
    def on_listWidget_itemClicked(self, item):
        self.toggleBusy(True)
        if self.actionPencil.isChecked():
            checkBox = self.listWidget.itemWidget(item).checkBox
            checkBox.toggle()
            if checkBox.isChecked():
                self.pencilList.append(self.listWidget.row(item))
                self.actionPencilQueue.setVisible(True)
                self.actionPencilDownloads.setVisible(True)
                if youtubeservice.YouTubeDataService.currentUser != "":
                    self.actionPencilPlaylist.setVisible((self.windowId != "playlists"))
                    self.actionPencilFavourites.setVisible((self.windowId != "favorites"))
            else:
                self.pencilList.remove(self.listWidget.row(item))
                if len(self.pencilList) == 0:
                    self.actionPencilQueue.setVisible(False)
                    self.actionPencilDownloads.setVisible(False)
                    self.actionPencilPlaylist.setVisible(False)
                    self.actionPencilFavourites.setVisible(False)
        else:
            row = self.listWidget.row(item)
            self.videoinfo = self.yt.videoList[row]
            if self.windowId =="playlists":
                position = row + 1
                count = len(self.yt.videoList)
                videoInfoWindow = VideoInfoWindow(self, self.videoinfo, self.playlistId, position, count)
            elif self.windowId =="subscriptions":
                videoInfoWindow = VideoInfoWindow(self, self.videoinfo, subscribed=True)                
            else:
                videoInfoWindow = VideoInfoWindow(self, self.videoinfo)
            self.connect(videoInfoWindow, SIGNAL("dataLoaded(PyQt_PyObject)"), self.playingVideo1)
            self.connect(videoInfoWindow, SIGNAL("queueChanged(int)"), self.queueChanged)
            self.connect(videoInfoWindow, SIGNAL("downloadsChanged(int)"), self.downloadsChanged)
            self.listWidget.setCurrentRow(-1)
            self.actionPencil.setChecked(False)
            self.on_actionPencil_toggled(False)
            self.closeFilter()
            self.pencilList = []
        self.toggleBusy()
        
    @pyqtSignature("")
    def on_actionSubscribe_triggered(self):
        try:
            subscribed = youtubeservice.YouTubeDataService.addSubscription(self.username)
            if subscribed == True:
                QMaemo5InformationBox.information(self, "You have subscribed to \'%s\'!" % unicode(self.username, "utf-8"))
            else:
                QMaemo5InformationBox.information(self, subscribed)
        except:
            QMaemo5InformationBox.information(self, "Unable to add subscription")
        
    @pyqtSignature("")
    def on_actionSearch_triggered(self):
        searchDialog = youtubedialogs.SearchDialog(self)

    @pyqtSignature("")
    def on_actionNewSubscriptionVideos_triggered(self):
        self.showLatest()

    @pyqtSignature("")
    def on_actionAddComment_triggered(self):
        addCommentDialog = youtubedialogs.AddCommentDialog(self, self.video)
        self.connect(addCommentDialog, SIGNAL("accepted()"), self.reloadData)

    @pyqtSignature("")
    def on_actionGoToQueue_triggered(self):
        playbackQueueWindow = PlaybackQueueWindow(self)
        self.connect(playbackQueueWindow, SIGNAL("queueChanged(int)"), self.queueChanged)
        self.connect(playbackQueueWindow, SIGNAL("downloadsChanged(int)"), self.downloadsChanged)
        self.actionPencil.setChecked(False)
        self.on_actionPencil_toggled(False)
        
    @pyqtSignature("")
    def on_actionDownloads_triggered(self):
        downloadsWindow = DownloadsWindow(self)
        self.connect(downloadsWindow, SIGNAL("downloadsChanged(int)"), self.downloadsChanged)
        self.connect(downloadsWindow, SIGNAL("queueChanged(int)"), self.queueChanged)
        self.actionPencil.setChecked(False)
        self.on_actionPencil_toggled(False)

    @pyqtSignature("bool")
    def on_actionPencil_toggled(self, isChecked):
        if isChecked:
            for row in range(self.listWidget.count()):
                item = self.listWidget.item(row)
                item.setFlags(Qt.ItemIsEnabled)
                widget = self.listWidget.itemWidget(item)
                widget.showCheckBox()
        else:
            self.actionPencilQueue.setVisible(False)
            self.actionPencilDownloads.setVisible(False)
            self.actionPencilPlaylist.setVisible(False)
            self.actionPencilFavourites.setVisible(False)
            for row in range(self.listWidget.count()):
                item = self.listWidget.item(row)
                item.setFlags(Qt.ItemIsEnabled | Qt.ItemIsSelectable)
                widget = self.listWidget.itemWidget(item)
                widget.hideCheckBox()
            self.listWidget.setCurrentRow(-1)

    @pyqtSignature("")
    def on_actionPencilQueue_triggered(self):
        try:
            for row in self.pencilList:  
                video = self.yt.videoList[row]              
                youtubeservice.YouTubeQueueFeed.playlist.append(video)
            QMaemo5InformationBox.information(self, "Your chosen videos have been added to your playback queue")
            queueLength = len(youtubeservice.YouTubeQueueFeed.playlist)
            self.queueChanged(queueLength)
        except:
            QMaemo5InformationBox.information(self, "Unable to add videos to your playback queue")
        self.actionPencil.toggle()
            
    @pyqtSignature("")
    def on_actionPencilDownloads_triggered(self):
        try:
            for row in self.pencilList:  
                video = self.yt.videoList[row]              
                youtubeservice.YouTubeVideoDownloader.addTask(video)
            QMaemo5InformationBox.information(self, "Your chosen videos have been added to your download queue")
            taskQueueLength = len(youtubeservice.YouTubeVideoDownloader.taskQueue)
            self.downloadsChanged(taskQueueLength)
        except:
            QMaemo5InformationBox.information(self, "Unable to add videos to your download queue")
        self.actionPencil.toggle()

    @pyqtSignature("")
    def on_actionPencilPlaylist_triggered(self):
        videoList = []
        for row in self.pencilList:          
            videoList.append(self.yt.videoList[row])
        addToPlaylistDialog = youtubedialogs.AddToPlaylistDialog(self, videoList)
        self.actionPencil.toggle()

    @pyqtSignature("")
    def on_actionPencilFavourites_triggered(self):
        try:
            self.toggleBusy(True)
            for row in self.pencilList:
                videoId = self.yt.videoList[row].media.video_id.text
                youtubeservice.YouTubeDataService.addVideoToFavourites(videoId)
            QMaemo5InformationBox.information(self, "Your chosen videos have been added to your favorites")
        except:
            QMaemo5InformationBox.information(self, "Unable to add videos to your favorites")
        self.toggleBusy()
        self.actionPencil.toggle()

    def queueChanged(self, queueLength):
        if queueLength > 0:
            self.actionGoToQueue.setVisible(True)
            self.actionGoToQueue.setText("Queue (%d videos)" % queueLength)
        else:
            self.actionGoToQueue.setVisible(False)
        self.emit(SIGNAL("queueChanged(int)"), queueLength)
        
    def downloadsChanged(self,  taskQueueLength):
        if taskQueueLength > 0:
            self.actionDownloads.setVisible(True)
            self.actionDownloads.setText("Downloads (%d videos)" % taskQueueLength)
        else:
            self.actionDownloads.setVisible(False)
        self.emit(SIGNAL("downloadsChanged(int)"), taskQueueLength)
 
class YoutubeWindow(YoutubeWindows):
    def __init__(self, parent,  searchString="", searchOrder="relevance", username=None, windowId=None,
                 subscribed=False, video=None, related="video.related"):
        YoutubeWindows.__init__(self, parent)
        self.setWindowTitle("Most Viewed")
        self.windowId = windowId
        self.actionPlaying.setVisible(True)
        self.actionAddToQueue.setVisible(True)
        if youtubeservice.YouTubeDataService.currentUser != "":
            self.actionAddToFavourites.setVisible((self.windowId != "favorites"))
            self.actionAddToPlaylist.setVisible((self.windowId != "playlists"))
        self.actionPencil.setVisible(True)
        self.pencilList = []
        self.tline = "?time=today"
        self.feedType = "most_viewed"
        self.show()
        self.showfeedMenus(True)
        self.searchOrderList = ["relevance", "published", "viewCount", "rating"]
        self.timePushButtonToggled(0)
        self.searchPushButtonToggled(self.searchOrderList.index(searchOrder))        
        self.username = username

        if not searchString == "":
            self.searchQuery(searchString, searchOrder, username)
        elif windowId == "latestsubscribed":
            self.showLatest()
        elif windowId == "myvideos":
            self.showmyVideos()
        elif windowId == "favorites":
            self.showFavorites()
        elif windowId == "usersvideos":
            self.showUserVideos(username, subscribed)
        elif windowId == "relatedvideos":
            self.showRelated(video, related)
        elif windowId == "subscriptions":
            self.showSubscriptions()            
        elif windowId == "playlists":
            self.showPlaylists()            
        else:                        
            self.switchFT()
            self.showtimePushbutton(True)

    def lastitemcheck(self):
        if self.listWidget.count() == 1:
            self.listWidget.clear()

    def showPlaylists(self):
        self.windowId = "playlists"
        if self.username==None:
            self.setWindowTitle("My Playlists")
        else:
            self.setWindowTitle("%s\'s Playlists" % self.username)
        self.actionNewPlaylist.setVisible((self.username==None))
        self.actionEditPlaylist.setVisible((self.username==None))
        self.actionDeletePlaylist.setVisible((self.username==None))
        self.actionUnsubscribe.setVisible(False)
        self.actionPlayPlaylist.setVisible(False)
        self.actionRemoveFromPlaylist.setVisible(False)
        self.actionNewSubscriptionVideos.setVisible(False)
        self.actionAddToPlaylist.setVisible(False)
        self.MyAcctButtonsToggled(2)
        self.showMyAcctButtons(True)
        self.actionPencil.setVisible(False)
        self.connect(self.actionEditPlaylist, SIGNAL("triggered()"), self.editPlaylist)
        self.connect(self.actionDeletePlaylist, SIGNAL("triggered()"), self.deletePlaylist)        
        self.subyt = youtubeservice.YouTubePlaylistFeed(self, self.username)
        self.connect(self.subyt, SIGNAL("dataLoaded(PyQt_PyObject)"), self.loadPlaylists)
        self.reloadData(True)
        self.toggleBusy(True)
        self.noresultsLabel.setVisible(False)

    def loadPlaylists(self, playlists):
        if self.clearwidgets:
            self.sublistWidget.clear()
            self.clearwidgets = False
        i = 1
        for playlist in playlists:
            item = QListWidgetItem(" %d. %s  (%s videos)" % (i, unicode(playlist.title.text, "utf-8"), playlist.count_hint.text))
            item.setTextAlignment(Qt.AlignLeft | Qt.AlignVCenter)
            self.sublistWidget.addItem(item)
            i += 1
        self.moreButton.setVisible(False)
        self.toggleBusy(subyt=True)
        self.listWidget.setVisible(False)
        self.sublistWidget.show()
        self.sublistWidget.setFocus()

    def editPlaylist(self):
        row = self.sublistWidget.currentRow()
        playlist = self.subyt.playlistFeed.entry[row]
        editPlaylistDialog = youtubedialogs.EditPlaylistDialog(self, playlist)
        self.sublistWidget.setCurrentRow(-1)
        self.connect(editPlaylistDialog, SIGNAL("accepted()"), self.reloadplaylist)
        
    def reloadplaylist(self):
        self.reloadData(True)

    def deletePlaylist(self):
        row = self.sublistWidget.currentRow()
        playlist = self.subyt.playlistFeed.entry[row]
        playlistId = playlist.playlist_id.text
        try:
            playlistDeleted = youtubeservice.YouTubeDataService.deletePlaylist(playlistId)
            if playlistDeleted == True:
                self.reloadData(True)
                QMaemo5InformationBox.information(self, "Your playlist: \'%s\' has been deleted" % unicode(playlist.title.text, "utf-8"))
            else:
                QMaemo5InformationBox.information(self, playlistDeleted)
        except:
            QMaemo5InformationBox.information(self, "Unable to delete your playlist")
        self.sublistWidget.setCurrentRow(-1)
        
    @pyqtSignature("QListWidgetItem*")
    def on_sublistWidget_itemClicked(self, item):
        row = self.sublistWidget.row(item)
        if self.windowId == "playlists":
            if self.subyt.playlistFeed.entry[row].count_hint.text != "0":
                playlistName = self.subyt.playlistFeed.entry[row].title.text
                playlistId = self.subyt.playlistFeed.entry[row].playlist_id.text
                self.showPlaylistsVideos(playlistName, playlistId)
                self.sublistWidget.setCurrentRow(-1)
            else:
                QMaemo5InformationBox.information(self, "No video in playlist")                
        else:
            subscriptionName = self.subyt.subscriptionsFeed.entry[row].username.text
            self.showUserVideos(subscriptionName, True)
            self.sublistWidget.setCurrentRow(-1)

    @pyqtSignature("")
    def on_actionNewPlaylist_triggered(self):
        newPlaylistDialog = youtubedialogs.NewPlaylistDialog(self)
        self.connect(newPlaylistDialog, SIGNAL("accepted()"), self.reloadplaylist)

    def showPlaylistsVideos(self, playlistName, playlistId):
        self.playlistName = playlistName
        self.playlistId = playlistId
        self.setWindowTitle("%s" % unicode(self.playlistName, "utf-8"))
        self.actionPlayPlaylist.setVisible(True)
        self.actionRemoveFromPlaylist.setVisible((self.username==None))
        self.actionAddToQueue.setVisible(True)
        self.actionAddToFavourites.setVisible(True)
        self.actionPencil.setVisible(True)
        self.actionNewPlaylist.setVisible(False)
        self.actionDeleteVideo.setVisible(False)
        self.actionNewSubscriptionVideos.setVisible(False)
        self.actionRemoveFromFavourites.setVisible(False)
        self.connect(self.actionRemoveFromPlaylist, SIGNAL("triggered()"), self.removeFromPlaylist)
        self.videoFeed = "http://gdata.youtube.com/feeds/api/playlists/%s" % playlistId
        self.connectyt(50)

    def removeFromPlaylist(self):
        row = self.listWidget.currentRow()
        video = self.yt.videoList[row]
        videoId = self.yt.videoList[row].id.text.split(":")[-1]
        try:
            removedFromPlaylist = youtubeservice.YouTubeDataService.removeVideoFromPlaylist(self.playlistId, videoId)
            if removedFromPlaylist == True:
                self.lastitemcheck()
                self.reloadData()
                QMaemo5InformationBox.information(self, "\'%s\' has been removed from your playlist" % unicode(video.media.title.text, "utf-8"))
            else:
                QMaemo5InformationBox.information(self, removedFromPlaylist)
        except:
            QMaemo5InformationBox.information(self, "Unable to remove video from your playlist")
        self.listWidget.setCurrentRow(-1)
        
    @pyqtSignature("")
    def on_actionPlayPlaylist_triggered(self):
        try:
            if self.videoPlaybackWindow.player.isPlaying():
                self.videoPlaybackWindow.close()
        except:
            pass
        
        try:
            if self.flashView.isVisible():
                self.stopFlash()
            self.videoPlaybackWindow = VideoPlaybackWindow(self, playlist=self.yt.videoList)
            self.connect(self.videoPlaybackWindow, SIGNAL("dataLoaded(PyQt_PyObject)"), self.videoinfWindow)
        except:
            QMaemo5InformationBox.information(self, "Unable to play playlist")
   
    def showSubscriptions(self):
        self.windowId = "subscriptions"
        self.setWindowTitle("My Subscriptions")
        self.actionUnsubscribe.setVisible(True)
        self.actionNewSubscriptionVideos.setVisible(True)
        self.actionPlayPlaylist.setVisible(False)
        self.actionRemoveFromPlaylist.setVisible(False)
        self.actionNewPlaylist.setVisible(False)
        self.actionEditPlaylist.setVisible(False)
        self.actionDeletePlaylist.setVisible(False)
        self.actionPencil.setVisible(False)
        self.MyAcctButtonsToggled(3)
        self.showMyAcctButtons(True)        
        self.connect(self.actionUnsubscribe, SIGNAL("triggered()"), self.removeSubscription)
        self.subyt = youtubeservice.YouTubeSubscriptionsFeed(self)
        self.connect(self.subyt, SIGNAL("dataLoaded(PyQt_PyObject)"), self.loadSubscriptions)
        self.reloadData(True)    
        self.toggleBusy(True)
        self.noresultsLabel.setVisible(False)
           
    def loadSubscriptions(self, subscriptionsList):
        if self.clearwidgets:
            self.sublistWidget.clear()
            self.clearwidgets = False
        i = 1
        for subscription in subscriptionsList:
            item = QListWidgetItem(" %d. %s" % (i, subscription.username.text))
            item.setTextAlignment(Qt.AlignLeft | Qt.AlignVCenter)
            self.sublistWidget.addItem(item)
            i += 1
        self.toggleBusy(subyt=True)
        self.moreButton.setVisible(False)
        self.listWidget.setVisible(False)
        self.sublistWidget.show()
        self.sublistWidget.setFocus()
        
    def removeSubscription(self):
        row = self.sublistWidget.currentRow()
        subscription = self.subyt.subscriptionsFeed.entry[row]
        for link in subscription.link:
            if link.rel == "self":
                subscription_uri = link.href
        try:
            unsubscribed = youtubeservice.YouTubeDataService.removeSubscription(subscription_uri)
            if unsubscribed == True:
                self.reloadData(True)
                QMaemo5InformationBox.information(self, "You have unsubscribed to \'%s\'" % unicode(subscription.username.text, "utf-8"))
            else:
                QMaemo5InformationBox.information(self, unsubscribed) 
        except:
            QMaemo5InformationBox.information(self, "Unable to remove your subscription")    
        self.sublistWidget.setCurrentRow(-1)
            
    def showMyAcctButtons(self, a=False):
        if a:
            self.showsearchPushbutton()
            self.showtimePushbutton()
        for b in (self.myVideosButton, self.favoritesButton, self.playlistButton, self.subscriptionsButton):
            b.setVisible(a)
            
    def MyAcctButtonsToggled(self, a=4):
        i = 0
        for button in (self.myVideosButton, self.favoritesButton, self.playlistButton, self.subscriptionsButton):
            if i == a:    
                button.setStyleSheet("margin: 8px 0 0 0; border-radius: 3px; background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #4ABEFF, stop: 0.8 #000000);")
            else:
                button.setStyleSheet("border: 1px solid rgb(120, 120, 120); margin: 8px 0 0 0; border-radius: 3px;")
            i += 1
    @pyqtSignature("")
    def on_myVideosButton_clicked(self):
        self.showmyVideos()

    @pyqtSignature("")   
    def on_favoritesButton_clicked(self):
        self.showFavorites()

    @pyqtSignature("")   
    def on_playlistButton_clicked(self):
        self.showPlaylists()

    @pyqtSignature("")   
    def on_subscriptionsButton_clicked(self):
        self.showSubscriptions()      

    def showRelated(self, video=None, related="video.related"):
        self.MyAcctButtonsToggled()
        self.showMyAcctButtons(not (self.timePushButton.isVisible() or self.searchPushButton.isVisible() or youtubeservice.YouTubeDataService.currentUser == ""))
        self.actionDeleteVideo.setVisible(False)
        self.actionRemoveFromFavourites.setVisible(False)
        if related == "video.related":
            self.setWindowTitle("Related Videos")
        else:
            self.setWindowTitle("Responses Videos")
        for link in video.link:
            if link.rel.split("#")[-1] == related:
                self.videoFeed = link.href
        self.connectyt()

    def showUserVideos(self, username=None, subscribed=False):
        self.setWindowTitle("%s\'s Videos" % username)
        self.MyAcctButtonsToggled(3)
        self.showMyAcctButtons(not (self.timePushButton.isVisible() or self.searchPushButton.isVisible() or youtubeservice.YouTubeDataService.currentUser == ""))
        self.actionUnsubscribe.setVisible(subscribed)
        self.actionSubscribe.setVisible(False)
        self.actionAddToFavourites.setVisible(True)
        self.actionAddToPlaylist.setVisible(True)
        self.actionDeleteVideo.setVisible(False)
        self.actionRemoveFromFavourites.setVisible(False)
        self.actionNewPlaylist.setVisible(False)
        self.actionEditPlaylist.setVisible(False)
        self.actionDeletePlaylist.setVisible(False)
        self.actionPlayPlaylist.setVisible(False)
        self.actionRemoveFromPlaylist.setVisible(False)
        if not subscribed:
            self.MyAcctButtonsToggled()
            self.actionSubscribe.setVisible(True)
            self.actionSubscribe.setText("Subscribe to \'%s\'" % unicode(username, "utf-8"))
        self.actionSearch.setText("Search user's videos")
        self.videoFeed = "http://gdata.youtube.com/feeds/api/users/%s/uploads" % username
        self.connectyt()

    def showFavorites(self):
        self.windowId = "favorites"
        self.setWindowTitle("Favorites Videos")
        self.actionAddToPlaylist.setVisible(True)
        self.actionRemoveFromFavourites.setVisible(True)
        self.actionDeleteVideo.setVisible(False)
        self.actionAddToFavourites.setVisible(False)
        self.actionPencilFavourites.setVisible(False)
        self.actionNewPlaylist.setVisible(False)
        self.actionEditPlaylist.setVisible(False)
        self.actionDeletePlaylist.setVisible(False)
        self.actionPlayPlaylist.setVisible(False)
        self.actionRemoveFromPlaylist.setVisible(False)
        self.actionNewSubscriptionVideos.setVisible(False)
        self.MyAcctButtonsToggled(1)
        self.showMyAcctButtons(True)
        self.connect(self.actionRemoveFromFavourites, SIGNAL("triggered()"), self.removeFromFavourites)        
        self.videoFeed = "http://gdata.youtube.com/feeds/api/users/default/favorites"
        self.connectyt()
        
    def removeFromFavourites(self):
        row = self.listWidget.currentRow()
        video = self.yt.videoList[row]
        videoId = video.media.video_id.text
        try:
            removedFromFavourites = youtubeservice.YouTubeDataService.removeVideoFromFavourites(videoId)
            if removedFromFavourites == True:
                self.lastitemcheck()
                self.reloadData()
                QMaemo5InformationBox.information(self, "\'%s\' has been removed from your favorites" % unicode(video.media.title.text, "utf-8"))
            else:
                QMaemo5InformationBox.information(self, removedFromFavourites)
        except:
            QMaemo5InformationBox.information(self, "Unable to remove video from your favorites")
        self.listWidget.setCurrentRow(-1)

    def showLatest(self):
        self.setWindowTitle("Latest Subscription Videos")
        self.videoFeed = "http://gdata.youtube.com/feeds/api/users/default/newsubscriptionvideos"
        self.MyAcctButtonsToggled(3)
        self.showMyAcctButtons(True)
        self.actionDeleteVideo.setVisible(False)
        self.actionUnsubscribe.setVisible(False)
        self.actionRemoveFromFavourites.setVisible(False)
        self.actionNewPlaylist.setVisible(False)
        self.actionEditPlaylist.setVisible(False)
        self.actionDeletePlaylist.setVisible(False)
        self.actionPlayPlaylist.setVisible(False)
        self.actionRemoveFromPlaylist.setVisible(False)
        self.connectyt()

    def showmyVideos(self):
        self.setWindowTitle("My Videos")
        self.actionDeleteVideo.setVisible(True)
        self.actionAddToFavourites.setVisible(True)
        self.actionUnsubscribe.setVisible(False)
        self.actionRemoveFromFavourites.setVisible(False)
        self.actionNewPlaylist.setVisible(False)
        self.actionEditPlaylist.setVisible(False)
        self.actionDeletePlaylist.setVisible(False)
        self.actionPlayPlaylist.setVisible(False)
        self.actionRemoveFromPlaylist.setVisible(False)
        self.actionNewSubscriptionVideos.setVisible(False)
        self.MyAcctButtonsToggled(0)
        self.showMyAcctButtons(True)
        self.videoFeed = "http://gdata.youtube.com/feeds/api/users/default/uploads"
        self.connectyt()

    def searchQuery(self, searchString="", searchOrder="relevance", username=None):
        if not searchString == "":
            self.windowId = "search"
            self.searchPushButtonToggled(self.searchOrderList.index(searchOrder))
            self.showsearchPushbutton(True)
            self.actionRemoveFromFavourites.setVisible(False)
            self.actionPencilFavourites.setVisible(False)
            self.actionNewPlaylist.setVisible(False)
            self.actionEditPlaylist.setVisible(False)
            self.actionDeletePlaylist.setVisible(False)
            self.actionSubscribe.setVisible(False)
            self.actionPlayPlaylist.setVisible(False)
            self.actionRemoveFromPlaylist.setVisible(False)
            self.actionAddToFavourites.setVisible((youtubeservice.YouTubeDataService.currentUser != ""))
            self.actionAddToPlaylist.setVisible((youtubeservice.YouTubeDataService.currentUser != ""))
            
            if youtubeservice.YouTubeDataService.currentUser == self.username:
                self.actionDeleteVideo.setVisible(True)
            self.setWindowTitle("Search Results")
            self.showtimePushbutton()
            self.searchString = searchString
            searchTerms = self.searchString.replace(" ", "+")
            if self.username == None:
                self.videoFeed = "http://gdata.youtube.com/feeds/api/videos?q=%s&safeSearch=none&orderby=%s" % (searchTerms, searchOrder)
            else:
                self.videoFeed = "http://gdata.youtube.com/feeds/api/videos?author=%s&q=%s&safeSearch=none&orderby=%s" % (self.username, searchTerms, searchOrder)
                self.actionSubscribe.setVisible(True)
            self.connectyt()
            if searchString in SearchSettings.recentSearches:
                SearchSettings.recentSearches.removeAll(searchString)
            if SearchSettings.recentSearches.count() > 30:
                SearchSettings.recentSearches.removeAt(30)
            SearchSettings.recentSearches.prepend(searchString)

    def showsearchPushbutton(self, a=False):
        if a:
            self.showMyAcctButtons()
            self.showtimePushbutton()
        for b in (self.searchPushButton, self.searchPushButton1, self.searchPushButton2, self.searchPushButton3):
            b.setVisible(a)
            
    def searchPushButtonToggled(self, a):
        i = 0
        for button in (self.searchPushButton, self.searchPushButton1, self.searchPushButton2, self.searchPushButton3):
            if i == a:    
                button.setStyleSheet("margin: 8px 0 0 0; border-radius: 3px; background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #4ABEFF, stop: 0.8 #000000);")
            else:
                button.setStyleSheet("border: 1px solid rgb(120, 120, 120); margin: 8px 0 0 0; border-radius: 3px;")
            i += 1        

    def showfeedMenus(self, a=False):        
        for b in (self.switchFeed, self.switchFeed1, self.switchFeed2, self.switchFeed3,
                  self.switchFeed4, self.switchFeed5, self.switchFeed6, self.switchFeed7, self.switchFeed8):
            b.setVisible(a)        

    def showtimePushbutton(self, a=False):
        if a:
            self.showMyAcctButtons()
            self.showsearchPushbutton()
        for b in (self.timePushButton, self.timePushButton1, self.timePushButton2, self.timePushButton3):
            b.setVisible(a)

    def timePushButtonToggled(self, a):
        i = 0
        for button in (self.timePushButton, self.timePushButton1, self.timePushButton2, self.timePushButton3):
            if i == a:    
                button.setStyleSheet("margin: 8px 0 0 0; border-radius: 3px; background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #4ABEFF, stop: 0.8 #000000);")
            else:
                button.setStyleSheet("border: 1px solid rgb(120, 120, 120); margin: 8px 0 0 0; border-radius: 3px;")
            i += 1

    @pyqtSignature("")
    def on_searchPushButton_clicked(self):
        self.searchQuery(self.searchString, "relevance")
        self.searchPushButtonToggled(0)

    @pyqtSignature("")   
    def on_searchPushButton1_clicked(self):
        self.searchQuery(self.searchString, "published")
        self.searchPushButtonToggled(1)

    @pyqtSignature("")   
    def on_searchPushButton2_clicked(self):
        self.searchQuery(self.searchString, "viewCount")
        self.searchPushButtonToggled(2)

    @pyqtSignature("")   
    def on_searchPushButton3_clicked(self):
        self.searchQuery(self.searchString, "rating")
        self.searchPushButtonToggled(3)
            
    @pyqtSignature("")
    def on_timePushButton_clicked(self):
        self.tline = "?time=today"
        self.switchFT()
        self.timePushButtonToggled(0)

    @pyqtSignature("")   
    def on_timePushButton1_clicked(self):
        self.tline = "?time=this_week"
        self.switchFT()
        self.timePushButtonToggled(1)

    @pyqtSignature("")   
    def on_timePushButton2_clicked(self):
        self.tline = "?time=this_month"
        self.switchFT()
        self.timePushButtonToggled(2)

    @pyqtSignature("")   
    def on_timePushButton3_clicked(self):
        self.tline = "?time=all_time"
        self.switchFT()
        self.timePushButtonToggled(3)

    @pyqtSignature("")
    def on_switchFeed_triggered(self):
        self.feedType = "top_rated"
        self.showtimePushbutton(True)
        self.switchFT()

    @pyqtSignature("")
    def on_switchFeed1_triggered(self):
        self.showtimePushbutton()
        self.showsearchPushbutton()
        self.tlinerecord = self.tline
        self.tline = ""
        self.feedType = "recently_featured"
        self.switchFT()
        self.tline = self.tlinerecord

    @pyqtSignature("")
    def on_switchFeed2_triggered(self):
        self.feedType = "most_discussed"
        self.switchFT()
        self.showtimePushbutton(True)

    @pyqtSignature("")
    def on_switchFeed3_triggered(self):
        self.feedType = "top_favorites"
        self.showtimePushbutton(True)
        self.switchFT()
        
    @pyqtSignature("")
    def on_switchFeed4_triggered(self):
        self.feedType = "most_responded"
        self.showtimePushbutton(True)
        self.switchFT()

    @pyqtSignature("")
    def on_switchFeed5_triggered(self):
        self.feedType = "most_recent"
        self.showtimePushbutton()
        self.showsearchPushbutton()
        self.tlinerecord = self.tline
        self.tline = ""
        self.switchFT()
        self.tline = self.tlinerecord
        
    @pyqtSignature("")
    def on_switchFeed6_triggered(self):
        self.feedType = "most_viewed"
        self.showtimePushbutton(True)
        self.switchFT()

    @pyqtSignature("")
    def on_switchFeed7_triggered(self):
        self.feedType = "most_shared"
        self.showtimePushbutton()
        self.showsearchPushbutton()
        self.tlinerecord = self.tline
        self.tline = ""
        self.switchFT()
        self.tline = self.tlinerecord

    @pyqtSignature("")
    def on_switchFeed8_triggered(self):
        self.feedType = "on_the_web"
        self.showtimePushbutton()
        self.showsearchPushbutton()
        self.tlinerecord = self.tline
        self.tline = ""
        self.switchFT()
        self.tline = self.tlinerecord

    def switchFT(self):
        self.windowId = "feed"
        self.username = None
        feedTypes = { "most_viewed": "Most Viewed", "top_rated": "Top Rated",
                      "recently_featured": "Recently Featured", "most_discussed": "Most Discussed",
                      "top_favorites": "Top Favorites", "most_responded": "Most Responded",
                      "most_recent": "Most Recent", "most_shared": "Most Shared", "on_the_web": "Trending Videos" }
        self.actionDeleteVideo.setVisible(False)
        self.actionRemoveFromFavourites.setVisible(False)
        self.actionPencilFavourites.setVisible(False)
        self.actionSubscribe.setVisible(False)
        self.actionNewPlaylist.setVisible(False)
        self.actionEditPlaylist.setVisible(False)
        self.actionDeletePlaylist.setVisible(False)
        self.actionPlayPlaylist.setVisible(False)
        self.actionRemoveFromPlaylist.setVisible(False)
        self.showMyAcctButtons(not (self.timePushButton.isVisible() or self.searchPushButton.isVisible() or youtubeservice.YouTubeDataService.currentUser == ""))
        self.MyAcctButtonsToggled()
        self.actionAddToFavourites.setVisible((youtubeservice.YouTubeDataService.currentUser != ""))
        self.actionAddToPlaylist.setVisible((youtubeservice.YouTubeDataService.currentUser != ""))        
        self.setWindowTitle(feedTypes[self.feedType])
        location = YouTubeAPISettings.locationFilter
        category = YouTubeAPISettings.categoryFilter
        self.videoFeed = "http://gdata.youtube.com/feeds/api/standardfeeds/%s%s%s%s" % (location, self.feedType, category, self.tline)
        self.connectyt()

    def connectyt(self, maxresults=10):
        self.actionPencil.setVisible(True)
        self.info = False
        self.noresultsLabel.setVisible(False)
        self.yt = youtubeservice.YouTubeVideoFeed(self, self.videoFeed, maxresults=maxresults)
        self.connect(self.yt, SIGNAL("dataLoaded(PyQt_PyObject)"), self.addVideo)
        self.connect(self.yt, SIGNAL("thumbnailLoaded(PyQt_PyObject)"), self.addVideoThumbnail)
        self.connect(self.yt, SIGNAL("feedCompleted(bool)"), self.toggleBusy)
        self.connect(self.listWidget.verticalScrollBar(), SIGNAL("valueChanged(int)"), self.getMoreResults)
        self.reloadData()
        self.listWidget.setFocus()

class PlaybackQueueWindow(YoutubeWindows):
    def __init__(self, parent):
        YoutubeWindows.__init__(self, parent)
        self.setWindowTitle("My Playback Queue")
        self.actionPlayQueue.setVisible(True)
        self.actionClearQueue.setVisible(True)
        if youtubeservice.YouTubeDataService.currentUser != "":
            self.actionSaveQueue.setVisible(True)
            self.actionAddToFavourites.setVisible(True)
            self.actionAddToPlaylist.setVisible(True)
        self.actionRemoveFromQueue.setVisible(True)
        self.actionGoToQueue.setVisible(False)
        self.connect(self.actionRemoveFromQueue, SIGNAL("triggered()"), self.removeFromQueue)
        self.show()
        self.yt = youtubeservice.YouTubeQueueFeed(self)
        self.connect(self.yt, SIGNAL("dataLoaded(PyQt_PyObject)"), self.addVideo)
        self.connect(self.yt, SIGNAL("thumbnailLoaded(PyQt_PyObject)"), self.addVideoThumbnail)
        self.connect(self.yt, SIGNAL("feedCompleted(bool)"), self.toggleBusy)
        self.loadData()
        
    @pyqtSignature("QListWidgetItem*")
    def on_listWidget_itemClicked(self, item):
        row = self.listWidget.row(item)
        self.videoinfo = youtubeservice.YouTubeQueueFeed.playlist[row]
        videoInfoWindow = VideoInfoWindow(self, self.videoinfo)
        self.connect(videoInfoWindow, SIGNAL("dataLoaded(PyQt_PyObject)"), self.playingVideo1)
        self.connect(videoInfoWindow, SIGNAL("downloadsChanged(int)"), self.downloadsChanged)
        self.listWidget.setCurrentRow(-1)
        self.closeFilter()            

    def queueChanged(self, queueLength):
        pass

    def addToPlaylist(self):
        row = self.listWidget.currentRow()
        video = youtubeservice.YouTubeQueueFeed.playlist[row]
        addToPlaylistDialog = youtubedialogs.AddToPlaylistDialog(self, [video])
        self.listWidget.setCurrentRow(-1)

    def addToFavourites(self):
        row = self.listWidget.currentRow()
        video = youtubeservice.YouTubeQueueFeed.playlist[row]
        videoId = video.media.video_id.text
        try:
            addedToFavourites = youtubeservice.YouTubeDataService.addVideoToFavourites(videoId)
            if addedToFavourites == True:
                QMaemo5InformationBox.information(self, "\'%s\' has been added to your favorites" % unicode(video.media.title.text, "utf-8"))
            else:
                QMaemo5InformationBox.information(self, addedToFavourites)
        except:
            QMaemo5InformationBox.information(self, "Unable to add video to your favorites")
        self.listWidget.setCurrentRow(-1)

    def removeFromQueue(self):
        row = self.listWidget.currentRow()
        video = youtubeservice.YouTubeQueueFeed.playlist[row]
        videoTitle = video.media.title.text
        try:
            youtubeservice.YouTubeQueueFeed.playlist.remove(video)
            self.listWidget.setRowHidden(row, True)
            QMaemo5InformationBox.information(self, "\'%s\' has been removed from your playback queue" % unicode(videoTitle, "utf-8"))
            queueLength = len(youtubeservice.YouTubeQueueFeed.playlist)
            if queueLength == 0:
                self.actionClearQueue.setVisible(False)
                self.actionSaveQueue.setVisible(False)
                self.actionPlayQueue.setVisible(False)
            self.emit(SIGNAL("queueChanged(int)"), queueLength)
        except:
            QMaemo5InformationBox.information(self, "Unable to remove video from your playback queue")
        self.listWidget.setCurrentRow(-1)
        
    @pyqtSignature("")
    def on_actionPlayQueue_triggered(self):
        try:
            if self.videoPlaybackWindow.player.isPlaying():
                self.videoPlaybackWindow.close()
        except:
            pass
        
        try:
            playlist = youtubeservice.YouTubeQueueFeed.playlist
            self.videoPlaybackWindow = VideoPlaybackWindow(self, playlist = playlist)
            self.connect(self.videoPlaybackWindow, SIGNAL("dataLoaded(PyQt_PyObject)"), self.videoinfWindow)
        except:
            QMaemo5InformationBox.information(self, "Unable to play your playback queue")

    @pyqtSignature("")
    def on_actionClearQueue_triggered(self):
        try:
            youtubeservice.YouTubeQueueFeed.playlist = []
            self.listWidget.clear()
            QMaemo5InformationBox.information(self, "Your playback queue has been cleared")
            self.actionClearQueue.setVisible(False)
            self.actionSaveQueue.setVisible(False)
            self.actionPlayQueue.setVisible(False)
            queueLength = len(youtubeservice.YouTubeQueueFeed.playlist)
            self.emit(SIGNAL("queueChanged(int)"), queueLength)
            self.close()
        except:
            QMaemo5InformationBox.information(self, "Unable to clear your playback queue")

    @pyqtSignature("")
    def on_actionSaveQueue_triggered(self):
        videoQueue = youtubeservice.YouTubeQueueFeed.playlist
        saveQueueAsPlaylistDialog = youtubedialogs.NewPlaylistDialog(self, videoQueue)

class DownloadsWindow(YoutubeWindows):
    def __init__(self, parent):
        YoutubeWindows.__init__(self, parent)
        self.setWindowTitle("Downloads")
        self.actionRemoveFromQueue.setVisible(True)
        self.actionRemoveFromQueue.setText("Cancel download")
        self.actionClearQueue.setVisible(True)
        self.actionClearQueue.setText("Clear downloads")
        self.actionPauseDownload.setVisible(True)
        self.actionResumeDownload.setVisible(True)
        self.actionDownloads.setVisible(False)
        self.actionGoToQueue.setVisible(False)
        self.actionResumeDownloads.setVisible(True)
        self.actionPauseDownloads.setVisible(True)
        self.filterGroup = QActionGroup(self)
        self.filterGroup.setExclusive(True)
        self.actionQueued = QAction("Current", self.filterGroup)
        self.actionQueued.setCheckable(True)
        self.actionQueued.setChecked(False)
        self.actionAll = QAction("All", self.filterGroup)
        self.actionAll.setCheckable(True)
        self.actionAll.setChecked(True)
        self.actionCompleted = QAction("Completed", self.filterGroup)
        self.actionCompleted.setCheckable(True)
        self.actionCompleted.setChecked(False)
        self.menuMenu.addActions(self.filterGroup.actions())
        self.connect(self.filterGroup, SIGNAL("selected(QAction*)"), self.filterDownloads)
        self.connect(self.actionRemoveFromQueue,  SIGNAL("triggered()"),  self.cancelDownload)
        self.connect(self.actionPauseDownload,  SIGNAL("triggered()"),  self.pauseDownload)
        self.connect(self.actionResumeDownload,  SIGNAL("triggered()"),  self.resumeDownload)
        self.show()
        self.yt = youtubeservice.YouTubeDownloadQueueFeed(self)
        self.connect(self.yt, SIGNAL("dataLoaded(PyQt_PyObject)"), self.addVideo)
        self.connect(self.yt, SIGNAL("thumbnailLoaded(PyQt_PyObject)"), self.addVideoThumbnail)
        self.connect(self.yt, SIGNAL("feedCompleted(bool)"), self.toggleBusy)
        self.loadData()
        for task in youtubeservice.YouTubeVideoDownloader.taskQueue:
            self.connect(task, SIGNAL("downloadUpdated()"), self.updateDownloadStatus)
            self.connect(task, SIGNAL("statusChanged()"), self.updateTaskStatus)
            
    def filterDownloads(self, filter):
        if filter is self.actionAll:
            for row in range(self.listWidget.count()):
                self.listWidget.setRowHidden(row, False)
        elif filter is self.actionQueued:
            for row in range(self.listWidget.count()):
                task = youtubeservice.YouTubeVideoDownloader.taskQueue[row]
                if task.getStatus() == "completed":
                    self.listWidget.setRowHidden(row, True)
                else:
                    self.listWidget.setRowHidden(row, False)
        else:
            for row in range(self.listWidget.count()):
                task = youtubeservice.YouTubeVideoDownloader.taskQueue[row]
                if task.getStatus() == "completed":
                    self.listWidget.setRowHidden(row, False)
                else:
                    self.listWidget.setRowHidden(row, True)
        self.filterMenu(filter)
        self.listWidget.setCurrentRow(-1)
        
    def filterMenu(self, filter):
        if filter is self.actionAll:
            for action in (self.actionPauseDownload, self.actionResumeDownload, self.actionRemoveFromQueue, self.actionResumeDownloads, self.actionPauseDownloads, self.actionClearQueue):
                action.setVisible(True)
        elif filter is self.actionQueued:
            for action in (self.actionPauseDownload, self.actionResumeDownload, self.actionRemoveFromQueue, self.actionResumeDownloads, self.actionPauseDownloads):
                action.setVisible(True)
            self.actionClearQueue.setVisible(False)
        else:
            for action in (self.actionPauseDownload, self.actionResumeDownload, self.actionRemoveFromQueue, self.actionResumeDownloads, self.actionPauseDownloads, self.actionClearQueue):
                action.setVisible(False)
                
    def addVideo(self, videoData):
        size = QSize()
        size.setHeight(140)
        item = QListWidgetItem()
        item.setSizeHint(size)
        self.itemWidget = delegatewidgets.VideoDownloadItemWidget(self.listWidget, videoData)
        self.listWidget.addItem(item)
        self.listWidget.setItemWidget(item, self.itemWidget)
        row = self.listWidget.row(item)
        task = youtubeservice.YouTubeVideoDownloader.taskQueue[row]
        self.updateTaskStatus(task)

    def addVideoThumbnail(self, videoData):
        row, thumbnail = videoData
        item = self.listWidget.item(row)
        thumbLabel = self.listWidget.itemWidget(item).thumbLabel
        try:
            thumbLabel.setPixmap(thumbnail)
        except:
            thumbLabel.setPixmap(QPixmap(":/images/ui-images/thumbnail.png"))

    def filterList(self, event):
        if event.key() not in (Qt.Key_Left, Qt.Key_Right, Qt.Key_Up, Qt.Key_Down, Qt.Key_Backspace, Qt.Key_Return, Qt.Key_Control):
            self.toolBar.setVisible(True)
            self.lineFilter.insert(event.text())
        if event.key() == Qt.Key_Backspace:
            self.lineFilter.backspace()
        for row in range(self.listWidget.count()):
            item = self.listWidget.item(row)
            itemWidget = self.listWidget.itemWidget(item)
            title = itemWidget.titleLabel.text()
            if not title.contains(self.lineFilter.text(), Qt.CaseInsensitive):
                self.listWidget.setRowHidden(row, True)
            else:
                self.listWidget.setRowHidden(row, False)
            if self.lineFilter.text() == "":
                self.toolBar.setVisible(False)                
        
    @pyqtSignature("QListWidgetItem*")
    def on_listWidget_itemClicked(self, item):
        row = self.listWidget.row(item)
        self.videoinfo = youtubeservice.YouTubeVideoDownloader.taskQueue[row].video
        videoInfoWindow = VideoInfoWindow(self, self.videoinfo)
        self.connect(videoInfoWindow, SIGNAL("dataLoaded(PyQt_PyObject)"), self.playingVideo1)        
        self.connect(videoInfoWindow, SIGNAL("queueChanged(int)"), self.queueChanged)
        self.listWidget.setCurrentRow(-1)
        self.closeFilter()
            
    @pyqtSignature("")
    def on_actionClearQueue_triggered(self):
        if youtubeservice.YouTubeVideoDownloader.currentStatus == "active":
            QMaemo5InformationBox.information(self,  "Downloads are still in progress")
        else:
            try:
                youtubeservice.YouTubeVideoDownloader.clearTasks()
                self.listWidget.clear()
                QMaemo5InformationBox.information(self, "Your downloads have been cleared")
                self.actionClearQueue.setVisible(False)
                self.actionPlayQueue.setVisible(False)
                self.emit(SIGNAL("downloadsChanged(int)"), 0)
                self.close()
            except:
                QMaemo5InformationBox.information(self, "Unable to clear your downloads")
            
    def cancelDownload(self):
        row = self.listWidget.currentRow()
        youtubeservice.YouTubeVideoDownloader.cancelTask(row)
        task = youtubeservice.YouTubeVideoDownloader.taskQueue[row]
        if task.getStatus() != "completed":
            self.updateTaskStatus(task)
        self.listWidget.setCurrentRow(-1)
        
    def pauseDownload(self):
        row = self.listWidget.currentRow()
        youtubeservice.YouTubeVideoDownloader.pauseTask(row)
        task = youtubeservice.YouTubeVideoDownloader.taskQueue[row]
        if task.getStatus() != "completed":
            self.updateTaskStatus(task)
        self.listWidget.setCurrentRow(-1)
        
    def resumeDownload(self):
        row = self.listWidget.currentRow()
        youtubeservice.YouTubeVideoDownloader.resumeTask(row)
        task = youtubeservice.YouTubeVideoDownloader.taskQueue[row]
        if task.getStatus() != "completed":
            self.updateTaskStatus(task)
        self.listWidget.setCurrentRow(-1)
        
    @pyqtSignature("")
    def on_actionResumeDownloads_triggered(self):
        for row in range(self.listWidget.count()):
            youtubeservice.YouTubeVideoDownloader.resumeTask(row)
            task = youtubeservice.YouTubeVideoDownloader.taskQueue[row]
            if task.getStatus() != "completed":
                self.updateTaskStatus(task)
            
    @pyqtSignature("")
    def on_actionPauseDownloads_triggered(self):
        for row in range(self.listWidget.count()):
            youtubeservice.YouTubeVideoDownloader.pauseTask(row)
            task = youtubeservice.YouTubeVideoDownloader.taskQueue[row]
            if task.getStatus() != "completed":
                self.updateTaskStatus(task)

    def updateDownloadStatus(self):
        currentTask = youtubeservice.YouTubeVideoDownloader.currentTask
        row = youtubeservice.YouTubeVideoDownloader.taskQueue.index(currentTask)
        item = self.listWidget.item(row)
        widget = self.listWidget.itemWidget(item)
        try:
            if widget.speedLabel.text() != "completed":
                widget.progressBar.setValue(int(currentTask.progress*100))
                widget.speedLabel.setText("d/l speed: %.2f KB/s" % currentTask.speed)
                if currentTask.progress  == 1.0:
                    QMaemo5InformationBox.information(self,  "Download of \'%s\' is completed" % unicode(currentTask.getTitle(), "utf-8"))
                    widget.speedLabel.setText("completed")
                    self.filterDownloads(self.filterGroup.checkedAction())
        except AttributeError:
            pass
            
    def updateTaskStatus(self,  task = youtubeservice.YouTubeVideoDownloader.currentTask):
        try:
            row = youtubeservice.YouTubeVideoDownloader.taskQueue.index(task)
            item = self.listWidget.item(row)
            widget = self.listWidget.itemWidget(item)
            for status in ("queued",  "paused",  "completed",  "cancelled",  "failed"):
                if task.getStatus() == status:
                    widget.speedLabel.setText(status)
                widget.progressBar.setValue(int(task.progress*100))
        except ValueError:
            pass
        
    def toggleBusy(self, isBusy=False):
        self.setAttribute(Qt.WA_Maemo5ShowProgressIndicator, isBusy)
        self.loadingLabel.setVisible(isBusy)

class CommentsWindow(YoutubeWindows):
    def __init__(self, parent, video):
        YoutubeWindows.__init__(self, parent)
        self.setWindowTitle("Comments")
        if youtubeservice.YouTubeDataService.currentUser != "":
            self.actionAddComment.setVisible(True)
        self.actionGoToQueue.setVisible(False)
        self.show()
        self.video = video
        self.videoFeed = "http://gdata.youtube.com/feeds/api/videos/%s/comments?" % self.video.media.video_id.text
        self.yt = youtubeservice.YouTubeCommentsFeed(self, self.videoFeed)
        self.connect(self.yt, SIGNAL("dataLoaded(PyQt_PyObject)"), self.loadComments)
        self.connect(self.yt, SIGNAL("feedCompleted(bool)"), self.toggleBusy)
        self.connect(self.listWidget.verticalScrollBar(), SIGNAL("valueChanged(int)"), self.getMoreResults)
        self.loadData()

    def getMoreResults(self, scrollBarPos):
        if self.yt.isFinished() and self.yt.moreResults and scrollBarPos > (self.yt.results - 10) * 75:
            self.moreButton.setVisible(True)
        else:
            self.moreButton.setVisible(False)

    def filterList(self, event):
        pass

    def loadComments(self, comments):
        if self.clearwidgets:
            self.listWidget.clear()
            self.clearwidgets = False
        elif len(comments) < 1:
            item = QListWidgetItem("No comments")
            self.listWidget.addItem(item)  
        else:
            try:
                commentsNumber = unicode(len(comments))
                videoTitle = self.video.media.title.text
                size = QSize()
                for comment in comments:
                    item = QListWidgetItem()
                    item.setFlags(Qt.ItemIsEnabled)
                    self.listWidget.addItem(item)
                    commentWidget = delegatewidgets.CommentWidget(self.listWidget, comment.author[0].name.text, comment.content.text)
                    size.setHeight(commentWidget.height())
                    item.setSizeHint(size)
                    self.listWidget.setItemWidget(item, commentWidget)
            except AttributeError:
                pass

    @pyqtSignature("QListWidgetItem*")
    def on_listWidget_itemClicked(self, item):
        pass

    def queueChanged(self, queueLength):
        pass
        
    def downloadsChanged(self,  taskQueueLength):
        pass
